@@ -25,6 +25,17 @@
 #include <glom/libglom/sharedptr.h>
 #include <glom/libglom/data_structure/fieldtypes.h>
 
+typedef struct AvahiEntryGroup AvahiEntryGroup;
+typedef struct AvahiClient AvahiClient;
+
+namespace Gtk
+{
+  class Window;
+}
+
+namespace Glom
+{
+
 class ExceptionConnection : public std::exception
 {
 public:
@@ -102,6 +113,11 @@ public:
 
   static sharedptr<SharedConnection> get_and_connect();
 
+  /** This specifies that Glom should start its own database server instance for this database,
+   *  using the database files stored at the specified uri.
+   */
+  void set_self_hosted(const std::string& data_uri);
+
 
   void set_host(const Glib::ustring& value);
   void set_user(const Glib::ustring& value);
@@ -122,27 +138,82 @@ public:
    */
   float get_postgres_server_version();
 
+  /** Start a database server instance for the exisiting database files.
+   */
+  void start_self_hosting();
+
+  /** Stop the database server instance for the database files.
+   */
+  void stop_self_hosting();
+
+  /** Create new database files, for later use by their own  database server instance.
+   */
+  bool create_self_hosting();
+
   //Show the gda error in a dialog.
   static bool handle_error(bool cerr_only = false);
 
+
+  /** Check whether PostgreSQL is really available for self-hosting,
+   * in case the distro package has incorrect dependencies.
+   *
+   * @results True if everything is OK.
+   */
+  static bool check_postgres_is_available_with_warning();
+
+  /** Try to install postgres on the distro, though this will require a distro-specific 
+   * patch to the implementation.
+   */
+  static bool install_postgres(Gtk::Window* parent_window);
+
 protected:
   void on_sharedconnection_finished();
 
+  static bool create_text_file(const std::string& file_uri, const std::string& contents);
+
+  /** Examine ports one by one, starting at @a starting_port, in increasing order,
+   * and return the first one that is available.
+   */
+  static int discover_first_free_port(int start_port, int end_port);
+
+  /** Advertize self-hosting via avahi:
+   */
+  void avahi_start_publishing();
+  void avahi_stop_publishing();
+
+public:
+  void avahi_create_services(AvahiClient *c);
+protected:
+
+  typedef std::list<Glib::ustring> type_list_ports;
+  type_list_ports m_list_ports; //Network ports on which to try connecting to postgres.
+
   Glib::RefPtr<Gnome::Gda::Client> m_GdaClient;
   //Gnome::Gda::DataSourceInfo m_GdaDataSourceInfo;
 
+  bool m_self_hosting_active;
+  std::string m_self_hosting_data_uri;
 
   Glib::RefPtr<Gnome::Gda::Connection> m_refGdaConnection;
   guint m_sharedconnection_refcount;
   bool m_ready_to_connect;
-  Glib::ustring m_host, m_user, m_password, m_database;
+  Glib::ustring m_host, m_user, m_password, m_database, m_port;
   FieldTypes* m_pFieldTypes;
   float m_postgres_server_version;
 
+public:
+  AvahiEntryGroup* m_avahi_group;
+  AvahiClient* m_avahi_client;
+  GMainLoop* m_avahi_mainloop;
+  Glib::ustring m_avahi_service_name;
+
+private:
+
   static ConnectionPool* m_instance;
 
 };
 
+} //namespace Glom
 
 #endif //GLOM_CONNECTIONPOOL_H
 
